/*

*/
#include <windows.h>
#include <stdio.h>
#include "glut.h"
#include "glui.h"

#define rawImageWidth  1024
#define rawImageHeight  512
GLubyte rawImage1[rawImageHeight][rawImageWidth][3];
GLubyte rawImage2[rawImageHeight][rawImageWidth][3];

#define sliderInit 0.0
float sliderValue = sliderInit;  // this is set by the glui slider() callback function


/* GLUT callback function prototypes */
void display(void);
void reshape(int, int);
void slider(float);

int windowWidth;
int windowHeight;

/*  Read an image from a "raw" image file  */
void readRawImage(char* imageFile, GLubyte* imageArray )
{
   FILE* file;

   file = fopen(imageFile, "rb");
   if (!file)
   {
      printf("Unable to open file: %s\n", imageFile);
      exit(1);
   }
   // Now read the raw image file into imageArray
   fread(imageArray, rawImageHeight * rawImageWidth * 3, 1, file);
   fclose(file);
}


int main(int argc, char** argv)
{
   int window;
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
   glutInitWindowSize(rawImageWidth, rawImageHeight+20);
   glutInitWindowPosition(150, 150);
   window = glutCreateWindow("Day, Night");
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0, rawImageWidth, 0, rawImageHeight); /* viewing volume */
   glMatrixMode(GL_MODELVIEW);

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 10, 0, -10, 20, sliderInit, slider);

   // read the image data
   readRawImage("night.raw", (GLubyte*)rawImage1);
   readRawImage("earth.raw", (GLubyte*)rawImage2);

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


/* The display() callback function */
void display( void )
{
   int i, j;
   GLubyte r, g, b;
   float t = sliderValue;

   // specify the viewport
   glViewport( 0,  0, windowWidth, windowHeight-20); // left, bottom, width, height (in pixels)

   glClear(GL_COLOR_BUFFER_BIT);  /* clear the window */

   // Use the geometry pipline.to iterate through space
   // one point (essentially, one pixel) at a time.
   glBegin(GL_POINTS);
      for (i = 0; i < rawImageHeight; i++)
         for (j = 0; j < rawImageWidth; j++)
         {
            // For each pixel, we "lerp" the r, g, b values between the two images.
            r = (GLubyte)( (1-t)*rawImage1[i][j][0] + t*rawImage2[i][j][0] );
            g = (GLubyte)( (1-t)*rawImage1[i][j][1] + t*rawImage2[i][j][1] );
            b = (GLubyte)( (1-t)*rawImage1[i][j][2] + t*rawImage2[i][j][2] );
            glColor3ub(r, g, b);
            glVertex2i( j, i );    // plot the point (i.e., draw the pixel)
         }
         glEnd();

   glutSwapBuffers();  // glui needs double buffering!
}//display()


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback function */
void slider(float value)
{
   sliderValue = value;

   printf("We got a \"slider(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider()
