/*
   This program uses the geometry pipeline to paint what
   is essentially a two dimensional pixel image. It would
   be better to do this using the pixel pipeline.

   This version pre-computes the "image" data and stores
   it in a global array.
*/
#include <windows.h>
#include "glut.h"

#define SIZE 512
float image[SIZE][SIZE][3];  // array to hold pixel data

/* The display() callback function */
void display( void )
{
   int i, j;

   glClear(GL_COLOR_BUFFER_BIT);  /* clear the window */

   // This is geometric data, so it can be transformed
   //glRotatef(45.0, 0.0, 0.0, 1.0);

   // Use the geometry pipline.to iterate through space
   // one point (essentially, one pixel) at a time.
   glBegin(GL_POINTS);
      for (i = 0; i < SIZE; i++)
         for (j = 0; j < SIZE; j++)
         {
            glColor3f(image[i][j][0], image[i][j][1], image[i][j][2]);
            glVertex2f( i, j );    // plot the point (i.e., draw the pixel)
         }
         glEnd();

   glFlush(); /* clear buffers */

}//display()


int main(int argc, char** argv)
{
   unsigned grayScale;
   float gray;
   int i, j;

   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
   glutInitWindowSize(SIZE,SIZE);  /* 512 x 512 pixel window */
   glutInitWindowPosition(50,50);
   glutCreateWindow("Image");
   glutDisplayFunc(display);

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0, SIZE, 0, SIZE); /* viewing volume */
   glMatrixMode(GL_MODELVIEW);

   // compute the "image" data
   for (i = 0; i < SIZE; i++)
      for (j = 0; j < SIZE; j++)
      {
         grayScale = (i|j)%255;
         gray = ((float)grayScale)/255; // compute a "color"
         image[i][j][0] = gray;         // red component
         image[i][j][1] = gray;         // green component
         image[i][j][2] = gray;         // blue component
      }

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()
