/*
   This program uses the pixel pipeline to paint a
   two dimensional pixel image.
*/
#include <windows.h>
#include "glut.h"

#define SIZE 512
float image[SIZE][SIZE][3];  // array to hold pixel data

/* The display() callback function */
void display( void )
{
   glClear(GL_COLOR_BUFFER_BIT);  /* clear the window */

   // This is NOT geometric data, so it can't be transformed
   glRotatef(45.0, 0.0, 0.0, 1.0);  // this has no effect on the image

   // Use the pixel pipline to send the image data directly to the frame buffer.
   glRasterPos2i(0, 0);
   glDrawPixels(SIZE, SIZE, GL_RGB, GL_FLOAT, image);

   glFlush(); /* clear buffers */

}//display()


int main(int argc, char** argv)
{
   unsigned grayScale;
   double gray;
   int i, j;

   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
   glutInitWindowSize(SIZE,SIZE);  /* 512 x 512 pixel window */
   glutInitWindowPosition(50,50);
   glutCreateWindow("Image");
   glutDisplayFunc(display);

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0, SIZE, 0, SIZE); /* viewing volume */
   glMatrixMode(GL_MODELVIEW);

   // compute the "image" data
   for (i = 0; i < SIZE; i++)
      for (j = 0; j < SIZE; j++)
      {
         grayScale = (i|j)%255;
         gray = ((float)grayScale)/255; // compute a "color"
         image[i][j][0] = gray;         // red component
         image[i][j][1] = gray;         // green component
         image[i][j][2] = gray;         // blue component
      }

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()
