/*
   Based on  http://www.glprogramming.com/red/chapter09.html#name1
             http://www.morrowland.com/apron/tutorials/gl/gl_jpeg_loader.zip
             http://www.ijg.org/files/jpegsrc.v6b.tar.gz

   You need the two files jpeglib.h
                          jpeglib.lib

   See the makefile for the linker command that links this program with jpeglib.lib.
*/
#include <windows.h>
#include <stdio.h>
#include "glut.h"
#include "jpeglib.h"

#define jpegImageWidth  256
#define jpegImageHeight 256
GLubyte jpegImage[jpegImageHeight][jpegImageWidth][3];

GLuint texName;

/*  Read a texture from a "JPEG image file  */
void readRawImage(void)
{
   FILE* file;

	struct jpeg_decompress_struct cInfo;
	struct jpeg_error_mgr jerr;
	struct tImageJPG pImgData;
   int rowsRead;

   file = fopen("wood.jpg", "rb");
   if (!file)
   {
      printf("Unable to open file: %s\n", "wood.jpg");
      exit(1);
   }
   // Now decompress the JPEG file into the array jpegImage
	cInfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cInfo);
	jpeg_stdio_src(&cInfo, file);
   jpeg_read_header(&cInfo, TRUE);
	jpeg_start_decompress(&cInfo);
	pImgData.rowSpan = cInfo.image_width * cInfo.num_components;
	pImgData.sizeX   = cInfo.image_width;
	pImgData.sizeY   = cInfo.image_height;
	pImgData.data = (char*)jpegImage;
	rowsRead = 0;
   while (cInfo.output_scanline < cInfo.output_height)
	{
      char* ptr = (pImgData.data)+(rowsRead*(pImgData.rowSpan));
		rowsRead += jpeg_read_scanlines(&cInfo, &ptr, 1);
	}
	jpeg_finish_decompress(&cInfo);
	jpeg_destroy_decompress(&cInfo);
   fclose(file);
}


void init(void)
{
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel(GL_FLAT);
   glEnable(GL_DEPTH_TEST);

   readRawImage();
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

   glGenTextures(1, &texName);
   glBindTexture(GL_TEXTURE_2D, texName);

   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
                   GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                   GL_NEAREST);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, jpegImageWidth,
                jpegImageHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,
                jpegImage);
}


void display(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_TEXTURE_2D);
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
   glBindTexture(GL_TEXTURE_2D, texName);
   glBegin(GL_QUADS);
   glTexCoord2f(0.0, 0.0); glVertex3f(-2.0, -1.0, 0.0);
   glTexCoord2f(0.0, 1.0); glVertex3f(-2.0, 1.0, 0.0);
   glTexCoord2f(1.0, 1.0); glVertex3f(0.0, 1.0, 0.0);
   glTexCoord2f(1.0, 0.0); glVertex3f(0.0, -1.0, 0.0);

   glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, 0.0);
   glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 1.0, 0.0);
   glTexCoord2f(1.0, 1.0); glVertex3f(2.41421, 1.0, -1.41421);
   glTexCoord2f(1.0, 0.0); glVertex3f(2.41421, -1.0, -1.41421);
   glEnd();
   glFlush();
   glDisable(GL_TEXTURE_2D);
}


void reshape(int w, int h)
{
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(60.0, (GLfloat) w/(GLfloat) h, 1.0, 30.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glTranslatef(0.0, 0.0, -3.6);
}


void keyboard (unsigned char key, int x, int y)
{
   switch (key) {
      case 27:
         exit(0);
         break;
      default:
         break;
   }
}


int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(250, 250);
   glutInitWindowPosition(100, 100);
   glutCreateWindow(argv[0]);
   init();
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutMainLoop();
   return 0;
}
